package com.iamberry.app.tool.des;

import java.security.Key;
import java.security.MessageDigest;
import java.security.Security;

import javax.crypto.Cipher;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.iamberry.app.tool.log.RatFWLogger;

/**
 * MD5处理类
 * 注意：
 * 	1、在加密是不许要创建对象，但是在解密时必须创建对象
 * 	2、如果在加密时使用的秘钥和解密时使用的秘钥不符合，那么解析不正确,抛出异常:javax.crypto.BadPaddingException
 * @author 何秀刚
 */
@Component
public class MD5 {

	@Autowired
	private RatFWLogger logger;
	public void setInLongLogger(RatFWLogger logger) {
		this.logger = logger;
	}
	// md5字符数组
	private char[] chars = { '0', '1', '2', '3', '4', '5', '6', '7',
			'8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
	/**
	 * 将一个字符串转换为32位的字符串
	 */
	public String stringToMD5(String src) throws Exception {
		StringBuffer md5Str = new StringBuffer();

		byte[] bytes = src.getBytes();
		MessageDigest digest = MessageDigest.getInstance("MD5");
		byte[] newBytes = digest.digest(bytes);
		for (byte b : newBytes) {
			md5Str.append(chars[b >> 4 & 0x0F]);
			md5Str.append(chars[b & 0x0F]);
		}
		return md5Str.toString();
	}

	/** 字符串默认键值 */
	public static String strDefaultKey = "RATFWMD5";

	/** 加密工具 */
	public Cipher encryptCipher = null;

	/** 解密工具 */
	public Cipher decryptCipher = null;

	/**
	 * 将byte数组转换为表示16进制值的字符串， 如：byte[]{8,18}转换为：0813， 和public byte[]
	 * hexStr2ByteArr(String strIn) 互为可逆的转换过程
	 */
	public String byteArr2HexStr(byte[] arrB) throws Exception {
		int iLen = arrB.length;
		// 每个byte用两个字符才能表示，所以字符串的长度是数组长度的两倍
		StringBuffer sb = new StringBuffer(iLen * 2);
		for (int i = 0; i < iLen; i++) {
			int intTmp = arrB[i];
			// 把负数转换为正数
			while (intTmp < 0) {
				intTmp = intTmp + 256;
			}
			// 小于0F的数需要在前面补0
			if (intTmp < 16) {
				sb.append("0");
			}
			sb.append(Integer.toString(intTmp, 16));
		}
		return sb.toString();
	}

	/**
	 * 将表示16进制值的字符串转换为byte数组， 和public String byteArr2HexStr(byte[] arrB)
	 * 互为可逆的转换过程
	 */
	public byte[] hexStr2ByteArr(String strIn) throws Exception {
		byte[] arrB = strIn.getBytes();
		int iLen = arrB.length;

		// 两个字符表示一个字节，所以字节数组长度是字符串长度除以2
		byte[] arrOut = new byte[iLen / 2];
		for (int i = 0; i < iLen; i = i + 2) {
			String strTmp = new String(arrB, i, 2);
			arrOut[i / 2] = (byte) Integer.parseInt(strTmp, 16);
		}
		return arrOut;
	}

	/**
	 * 默认构造方法，使用默认密钥
	 * 
	 * @throws Exception
	 */
	public MD5() {
		this(strDefaultKey);
	}

	/**
	 * 指定密钥构造方法
	 * 
	 * @param strKey
	 *            指定的密钥
	 * @throws Exception
	 */
	public MD5(String strKey) {
		try {
			Security.addProvider(new com.sun.crypto.provider.SunJCE());
			Key key = getKey(strKey.getBytes());
			encryptCipher = Cipher.getInstance("DES");
			encryptCipher.init(Cipher.ENCRYPT_MODE, key);
			decryptCipher = Cipher.getInstance("DES");
			decryptCipher.init(Cipher.DECRYPT_MODE, key);
		} catch (Exception e) {
		}
	}

	/**
	 * 加密字节数组
	 */
	public byte[] encrypt(byte[] arrB) throws Exception {
		return encryptCipher.doFinal(arrB);
	}

	/**
	 * 加密字符串
	 */
	public String encrypt(String strIn) throws Exception {
		try {
			return byteArr2HexStr(encrypt(strIn.getBytes())).toUpperCase();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * 解密字节数组
	 */
	public byte[] decrypt(byte[] arrB) throws Exception {
		return decryptCipher.doFinal(arrB);
	}

	/**
	 * 解密字符串
	 */
	public String decrypt(String strIn) throws Exception {
		try {
			return new String(decrypt(hexStr2ByteArr(strIn)));
		} catch (Exception e) {
			logger.debug("加密和解密字符串不匹配:", e.getMessage());
			return "INLONGADMD5";
		}
	}

	/**
	 * 从指定字符串生成密钥，密钥所需的字节数组长度为8位 不足8位时后面补0，超出8位只取前8位
	 */
	private Key getKey(byte[] arrBTmp) throws Exception {
		// 创建一个空的8位字节数组（默认值为0）
		byte[] arrB = new byte[8];
		// 将原始字节数组转换为8位
		for (int i = 0; i < arrBTmp.length && i < arrB.length; i++) {
			arrB[i] = arrBTmp[i];
		}
		// 生成密钥
		Key key = new javax.crypto.spec.SecretKeySpec(arrB, "DES");
		return key;
	}
}
