package com.iamberry.wechat.service.wechat.dao;

import java.util.List;

import com.iamberry.wechat.core.entity.PaperBean;
import com.iamberry.wechat.core.entity.wx.ImageTextInfo;

public interface ImageTextInfoDAO {
	/**
	 * 添加图文素材
	 * --》添加ImageTextInfo对象
	 * @return 返回的是添加记录数
	 */
	public int addImageTextInfo(ImageTextInfo imageTextInfo); 
	
	/**
	 * 删除图文素材
	 * --》根据图文ID号来删除相应的图文素材
	 * @return 返回的是删除记录数
	 */
	public int delImageTextInfo(Long id);
	
	/**
	 * 修改图文素材
	 * --》根据ID来修改图文素材信息
	 * @return 返回的是修改记录数
	 */
	public int updateImageTextInfo(ImageTextInfo imageTextInfo);
	
	/**
	 * 按照id来查询查找图文素材
	 * 根据素材来出啊训
	 */
	public ImageTextInfo findImageTextInfoById(Long id);
	
	/**
	 * 按照公众账号来查询所有的图文素材
	 */
	public List<ImageTextInfo> findImageTextInfoBywxPublicNo(String wxPublicNo);
	
	/**
	 * 根据微信公众账号、页数、每页显示的记录来分页查询
	 */
	public List<ImageTextInfo> findImageTextInfo(String wxPublicNo, Integer pageNo, Integer selectNum);
	
	/**
	 * 根据微信公众号来查询该微信共有多少图文素材
	 */
	public int findImageTextInfoNo(String wxPublicNo);
	
	/**
	 * 分页查询图文素材
	 */
	public List<ImageTextInfo> selectImageTextInfoList(PaperBean bean);
	
	/**
	 * 根据公众账号查看全部素材
	 */
	public List<ImageTextInfo> selectAllImageTextList(String publicWxNo);
	
	/**
	 * 扫描其他表中有没有使用该素材的
	 */
	public Integer selectFirstAnswerByID(Integer imageTextId);
	public Integer selectKeyWordByID(Integer imageTextId);
	public Integer selectMenuKeyByID(Integer imageTextId);
	
}
